from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class InvoicingConfig(AppConfig):
    name = "basxconnect.invoicing"
    label = "basxconnect_invoicing"
    default_auto_field = "django.db.models.BigAutoField"
    verbose_name = _("BasxConnect Invoicing")



from django.apps import AppConfig


class MailerIntegrationConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "basxconnect.mailer_integration"

    def ready(self):
        import basxconnect.mailer_integration.signal_handlers  # noqa



# Generated by Django 3.2.5 on 2021-10-30 02:55

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("core", "0022_auto_20211018_1644"),
    ]

    operations = [
        migrations.AlterField(
            model_name="email",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="fax",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="historicallegalperson",
            name="id",
            field=models.BigIntegerField(
                auto_created=True, blank=True, db_index=True, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="historicalnaturalperson",
            name="id",
            field=models.BigIntegerField(
                auto_created=True, blank=True, db_index=True, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="historicalperson",
            name="id",
            field=models.BigIntegerField(
                auto_created=True, blank=True, db_index=True, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="historicalpersonassociation",
            name="id",
            field=models.BigIntegerField(
                auto_created=True, blank=True, db_index=True, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="note",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="person",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="phone",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="postal",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="relationship",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="relationshiptype",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="term",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="vocabulary",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="web",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
    ]



from basxbread.utils.urls import aslayout

import basxconnect.core.layouts.settings_layout as settings_layout


@aslayout
def relationshipssettings(request):
    return settings_layout.relationshipssettings(request)



import datetime

from django.db.models.signals import post_save
from django.dispatch import receiver
from django.utils import timezone

from basxconnect.core.models import Person
from basxconnect.mailer_integration.abstract.mailer import MailerPerson
from basxconnect.mailer_integration.models import Subscription
from basxconnect.mailer_integration.settings import MAILER


@receiver(post_save, sender=Person)
def update_subscription_active(sender, instance: Person, created, **kwargs):
    if hasattr(instance, "core_email_list"):
        for email in instance.core_email_list.all():
            if hasattr(email, "subscription"):
                if (
                    instance.active
                    and not instance.deleted
                    and email.subscription.status == "archived"
                ):
                    unarchive_subscription_from_mailer(email)
                elif (
                    not instance.active or instance.deleted
                ) and email.subscription.status != "archived":
                    archive_subscription_from_mailer(email)

            from dynamic_preferences.registries import global_preferences_registry

            if (
                global_preferences_registry.manager()[
                    "mailchimp__automatically_subscribe_new_persons"
                ]
                and not hasattr(email, "subscription")
                and instance.preferred_language != ""
                and instance.tags.filter(term="Newsletter").count() == 1
                and instance.history.last().history_date + datetime.timedelta(days=1)
                >= timezone.now()
            ):
                subscription = Subscription.objects.create(
                    email=email,
                    status="subscribed",
                    language=instance.preferred_language,
                )
                MAILER.add_person(MailerPerson.from_subscription(subscription))


def archive_subscription_from_mailer(email):
    email.subscription.status_before_archiving = email.subscription.status
    email.subscription.status = "archived"
    MAILER.delete_person(email.email)
    email.subscription.save()


def unarchive_subscription_from_mailer(email):
    email.subscription.status = email.subscription.status_before_archiving
    email.subscription.status_before_archiving = None
    MAILER.add_person(MailerPerson.from_subscription(email.subscription))
    email.subscription.save()



#!/usr/bin/env python

import os
import sys

import django
from django.conf import settings

if "test" in sys.argv:
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "basxconnect.core.tests.settings")
else:
    INSTALLED_APPS = [
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "django.contrib.sites",
        "basxbread",
        "djmoney",
        "basxconnect.core",
        "basxconnect.contributions",
        "basxconnect.mailer_integration",
        "basxconnect.invoicing",
        "basxconnect.projects",
    ]
    settings.configure(
        DEBUG=True,
        USE_TZ=True,
        USE_I18N=True,
        DATABASES={
            "default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}
        },
        MIDDLEWARE_CLASSES=(),
        SITE_ID=1,
        INSTALLED_APPS=INSTALLED_APPS,
        STATIC_URL="static/",
    )
    django.setup()

if __name__ == "__main__":
    from django.core.management import execute_from_command_line

    execute_from_command_line(sys.argv)



# Generated by Django 4.0.6 on 2022-12-15 05:48

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("core", "0034_term_disabled"),
    ]

    operations = [
        migrations.AlterField(
            model_name="email",
            name="type",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "emailtype"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="type_%(app_label)s_%(class)s_list",
                to="core.term",
                verbose_name="Type",
            ),
        ),
        migrations.AlterField(
            model_name="fax",
            name="type",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "phonetype"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="core.term",
                verbose_name="Type",
            ),
        ),
        migrations.AlterField(
            model_name="historicalnaturalperson",
            name="form_of_address",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"vocabulary__slug": "form_of_address"},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="core.term",
                verbose_name="Form of address",
            ),
        ),
        migrations.AlterField(
            model_name="historicalnaturalperson",
            name="gender",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"vocabulary__slug": "gender"},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="core.term",
                verbose_name="Gender",
            ),
        ),
        migrations.AlterField(
            model_name="historicalnaturalperson",
            name="salutation",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"vocabulary__slug": "salutation"},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="core.term",
                verbose_name="Salutation",
            ),
        ),
        migrations.AlterField(
            model_name="historicalnaturalperson",
            name="title",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"vocabulary__slug": "title"},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="core.term",
                verbose_name="Title",
            ),
        ),
        migrations.AlterField(
            model_name="naturalperson",
            name="form_of_address",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "form_of_address"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="core.term",
                verbose_name="Form of address",
            ),
        ),
        migrations.AlterField(
            model_name="naturalperson",
            name="gender",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "gender"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="gender_persons",
                to="core.term",
                verbose_name="Gender",
            ),
        ),
        migrations.AlterField(
            model_name="naturalperson",
            name="salutation",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "salutation"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="core.term",
                verbose_name="Salutation",
            ),
        ),
        migrations.AlterField(
            model_name="naturalperson",
            name="title",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "title"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="title_persons",
                to="core.term",
                verbose_name="Title",
            ),
        ),
        migrations.AlterField(
            model_name="phone",
            name="type",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "phonetype"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="core.term",
                verbose_name="Type",
            ),
        ),
        migrations.AlterField(
            model_name="postal",
            name="type",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "addresstype"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="type_%(app_label)s_%(class)s_list",
                to="core.term",
                verbose_name="Type",
            ),
        ),
        migrations.AlterField(
            model_name="web",
            name="type",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "urltype"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="type_%(app_label)s_%(class)s_list",
                to="core.term",
                verbose_name="Type",
            ),
        ),
    ]



from django.utils.translation import gettext_lazy as _
from djmoney.settings import CURRENCY_CHOICES
from dynamic_preferences.preferences import Section
from dynamic_preferences.registries import global_preferences_registry
from dynamic_preferences.types import ChoicePreference

contributions = Section("contributions", _("Contributions"))


@global_preferences_registry.register
class PreferredCurrency(ChoicePreference):
    section = contributions
    name = "currency"
    default = "EUR"
    verbose_name = _("Preferred Currency")
    choices = CURRENCY_CHOICES



# Generated by Django 4.1.7 on 2023-04-08 09:41

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("basxconnect_invoicing", "0002_alter_invoice_note"),
    ]

    operations = [
        migrations.AddField(
            model_name="invoice",
            name="receipt_sent",
            field=models.DateField(blank=True, null=True, verbose_name="Receipt sent"),
        ),
    ]



import collections
from typing import List, Union

import htmlgenerator as hg
from basxbread import layout
from basxbread.layout import ObjectFieldLabel, ObjectFieldValue
from basxbread.layout.components.icon import Icon
from basxbread.layout.components.modal import modal_with_trigger
from basxbread.utils import ModelHref
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
from htmlgenerator import Lazy

R = layout.grid.Row
C = layout.grid.Col
F = layout.forms.FormField


def person_metadata(model):
    return tiling_col(
        # we need this to take exactly as much space as a real header
        hg.DIV("", style="margin-bottom:3.25rem;"),
        display_field_label_and_value("personnumber"),
        display_field_label_and_value("maintype"),
        display_field_label_and_value("type"),
        display_label_and_value(_("Status"), active_toggle()),
        display_label_and_value(
            _("Changed"),
            hg.BaseElement(
                ObjectFieldValue("history.first.history_date.date"),
                " / ",
                hg.C("object.history.first.history_user"),
            ),
        ),
        display_label_and_value(
            _("Created"),
            hg.BaseElement(
                ObjectFieldValue("history.last.history_date.date"),
                " / ",
                hg.C("object.history.last.history_user"),
            ),
        ),
        open_modal_popup_button(
            _("Meta data"), model, f"{model._meta.model_name}_ajax_edit_metadata"
        ),
        style="border-left: none;",
    )


def active_toggle():
    toggle = layout.toggle.Toggle(
        None, _("Inactive"), _("Active"), style="margin-top:-1rem; margin-bottom:0;"
    )
    toggle.input.attributes["id"] = "person_active_toggle2"
    toggle.input.attributes["hx_trigger"] = "change"
    toggle.input.attributes["hx_post"] = hg.F(
        lambda c: reverse_lazy("core.person.togglestatus", args=[c["object"].pk])
    )
    toggle.input.attributes["checked"] = hg.F(lambda c: c["object"].active)
    toggle.label.attributes["_for"] = toggle.input.attributes["id"]
    return hg.DIV(toggle)


def grid_inside_tab(*elems, **attrs):
    attrs = collections.defaultdict(str, attrs or {})
    attrs["style"] += " padding-left: 1rem; padding-right: 1rem"
    return layout.grid.Grid(*elems, **attrs)


def tile_col_edit_modal(
    heading, modal_view: type, action: str, icon: Icon, fields: List[str]
):
    displayed_fields = [display_field_label_and_value(field) for field in fields]
    return tile_col_edit_modal_displayed_fields(
        heading, modal_view, action, icon, displayed_fields
    )


def tile_col_edit_modal_displayed_fields(
    heading, model: type, action: str, icon: Icon, displayed_fields: List
):
    return tile_with_icon(
        icon,
        hg.BaseElement(
            hg.H4(heading),
            *displayed_fields,
            open_modal_popup_button(heading, model, action),
        ),
    )


def open_modal_popup_button(heading, model, action):
    return R(
        C(
            modal_with_trigger(
                create_modal(heading, model, action),
                layout.button.Button,
                _("Edit"),
                buttontype="tertiary",
                icon="edit",
            ),
            style="margin-top: 1.5rem;",
        )
    )


def create_modal(heading, model: Union[type, Lazy], action: str):
    modal = layout.modal.Modal.with_ajax_content(
        heading=heading,
        url=ModelHref(
            model,
            action,
            kwargs={"pk": hg.F(lambda c: c["object"].pk)},
            query={"asajax": True},
        ),
        submitlabel=_("Save"),
    )
    return modal


def tile_with_icon(icon, *content):
    return tiling_col(R(C(icon, width=2), C(*content)))


def display_field_label_and_value(field):
    return display_label_and_value(ObjectFieldLabel(field), ObjectFieldValue(field))


def display_label_and_value(label, value):
    return R(
        C(
            hg.DIV(
                label,
                style="font-weight: bold;",
            ),
            width=6,
        ),
        C(value),
        style="padding-bottom: 1.5rem;",
    )


def tiling_col(*elems, **attrs):
    attrs = collections.defaultdict(str, attrs or {})
    attrs["_class"] += " tile tile-col theme-white"
    return C(*elems, **attrs)



# Generated by Django 3.2.5 on 2021-10-29 13:55

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("contributions", "0002_alter_contribution_currency"),
    ]

    operations = [
        migrations.AlterField(
            model_name="contribution",
            name="_import",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="contributions",
                to="contributions.contributionimport",
            ),
        ),
    ]



import datetime

import htmlgenerator as hg
from basxbread import layout, menu, utils, views
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _

from . import models
from .views import send_invoice, send_receipt


def invoice_download(context):
    from dynamic_preferences.registries import global_preferences_registry

    invoice_template = global_preferences_registry.manager()[
        "invoicing__default_invoice_template"
    ]
    return invoice_template.generate_document_url(context["row"], pdf=True)


def receipt_download(context):
    from dynamic_preferences.registries import global_preferences_registry

    receipt_template = global_preferences_registry.manager()[
        "invoicing__default_receipt_template"
    ]
    return receipt_template.generate_document_url(context["row"], pdf=True)


invoice_form = layout.grid.Grid(
    layout.grid.Row(
        layout.grid.Col(
            layout.tile.Tile(
                hg.If(
                    hg.C("object"),
                    hg.H3(
                        hg.C("object.total_amount"),
                        hg.DIV(
                            hg.BaseElement(
                                layout.ObjectFieldValue("net_amount"),
                                " ",
                                _("Net"),
                            ),
                            style="font-size: 0.75rem",
                        ),
                        hg.If(
                            hg.C("object.vat"),
                            hg.DIV(
                                hg.BaseElement(
                                    "+ ",
                                    layout.ObjectFieldValue("vat_amount"),
                                    " ",
                                    layout.ObjectFieldLabel("vat"),
                                ),
                                style="font-size: 0.75rem",
                            ),
                        ),
                        hg.If(
                            hg.C("object.wht"),
                            hg.DIV(
                                hg.BaseElement(
                                    "- ",
                                    layout.ObjectFieldValue("wht_amount"),
                                    " ",
                                    layout.ObjectFieldLabel("wht"),
                                ),
                                style="font-size: 0.75rem",
                            ),
                        ),
                    ),
                    hg.H3(_("New"), " ", models.Invoice._meta.verbose_name),
                ),
                hg.DIV(
                    layout.forms.FormField("number", no_wrapper=True),
                    layout.forms.FormField("client", no_wrapper=True),
                    layout.forms.FormField("paymenttype", no_wrapper=True),
                    style="display: flex; justify-content: space-between",
                ),
                hg.DIV(
                    layout.forms.FormField(
                        "note",
                        style="margin-top: 1rem; width: 100%",
                        inputelement_attrs={"rows": 4},
                    ),
                ),
                light=True,
            ),
            layout.forms.Formset.as_inline_datatable(
                "items", ["project", "description", "amount"]
            ),
            width=10,
        ),
        layout.grid.Col(
            layout.tile.Tile(
                layout.grid.Row(
                    layout.grid.Col(
                        hg.DIV(_("Dates"), style="margin-bottom: 1rem"),
                        layout.forms.FormField("created"),
                        layout.forms.FormField("invoice_sent"),
                        layout.forms.FormField("payed"),
                        layout.forms.FormField("receipt_sent"),
                        layout.forms.FormField("cancelled", style="margin-top: 3rem"),
                    ),
                    layout.grid.Col(
                        hg.DIV(
                            _("Currency and taxes"),
                            style="margin-bottom: 1rem",
                        ),
                        layout.forms.FormField("currency"),
                        layout.forms.FormField("amounts_include_vat"),
                        layout.forms.FormField("vat"),
                        layout.forms.FormField("wht"),
                    ),
                ),
                light=True,
            ),
            width=6,
        ),
    ),
    style="margin-bottom: 4rem",
)

urlpatterns = [
    *utils.default_model_paths(
        models.Invoice,
        browseview=views.BrowseView._with(
            queryset=models.Invoice.objects.get_active(),
            columns=[
                "created",
                "number",
                "client",
                "total_amount",
                layout.datatable.DataTableColumn(
                    layout.ObjectFieldLabel("invoice_sent", hg.C("object_list").model),
                    hg.If(
                        hg.C("row").invoice_sent,
                        hg.C("row").invoice_sent,
                        layout.modal.Modal.with_ajax_content(
                            hg.format(
                                _("Send invoice {} to {}"),
                                hg.C("row").number,
                                hg.C("row").client,
                            ),
                            utils.ModelHref(
                                hg.C("row"), "sendinvoice", query={"asajax": True}
                            ),
                            submitlabel=_("Send invoice"),
                            size="md",
                            id=hg.format("send-invoice-{}", hg.C("row").id),
                        ).with_trigger(
                            layout.button.Button.from_link(
                                utils.Link(
                                    href="",
                                    label=_("Send invoice"),
                                    iconname="mail--all",
                                    permissions=[
                                        hg.F(
                                            lambda c: utils.permissionname(
                                                c["row"], "change"
                                            )
                                        )
                                    ],
                                ),
                                notext=True,
                                small=True,
                                buttontype="ghost",
                            )
                        ),
                    ),
                ),
                layout.datatable.DataTableColumn(
                    layout.ObjectFieldLabel("payed", hg.C("object_list").model),
                    hg.If(
                        hg.C("row").payed,
                        hg.C("row").payed,
                        layout.modal.Modal.with_ajax_content(
                            hg.C("row"),
                            utils.ModelHref(
                                hg.C("row"), "markpayed", query={"asajax": True}
                            ),
                            submitlabel=_("Mark payed"),
                            id=hg.format("mark-payed-{}", hg.C("row").id),
                        ).with_trigger(
                            layout.button.Button.from_link(
                                utils.Link(
                                    href="",
                                    label=_("Mark payed"),
                                    iconname="checkmark",
                                    permissions=[
                                        hg.F(
                                            lambda c: utils.permissionname(
                                                c["row"], "change"
                                            )
                                        )
                                    ],
                                ),
                                notext=True,
                                small=True,
                                buttontype="ghost",
                            )
                        ),
                    ),
                ),
                layout.datatable.DataTableColumn(
                    layout.ObjectFieldLabel("receipt_sent", hg.C("object_list").model),
                    hg.If(
                        hg.C("row").receipt_sent,
                        hg.C("row").receipt_sent,
                        layout.modal.Modal.with_ajax_content(
                            hg.format(
                                _("Send receipt for invoice {} to {}"),
                                hg.C("row").number,
                                hg.C("row").client,
                            ),
                            utils.ModelHref(
                                hg.C("row"), "sendreceipt", query={"asajax": True}
                            ),
                            submitlabel=_("Send receipt"),
                            size="md",
                            id=hg.format("send-receipt-{}", hg.C("row").id),
                        ).with_trigger(
                            layout.button.Button.from_link(
                                utils.Link(
                                    href="",
                                    label=_("Send receipt"),
                                    iconname="mail--all",
                                    permissions=[
                                        hg.F(
                                            lambda c: utils.permissionname(
                                                c["row"], "change"
                                            )
                                        )
                                    ],
                                ),
                                notext=True,
                                small=True,
                                buttontype="ghost",
                            )
                        ),
                    ),
                ),
                layout.datatable.DataTableColumn(
                    _("Invoice"),
                    hg.If(
                        hg.F(
                            lambda c: models.pref("default_invoice_template")
                            is not None
                        ),
                        layout.button.Button(
                            notext=True, icon="download", buttontype="ghost", small=True
                        ).as_href(hg.F(invoice_download)),
                        hg.SPAN(
                            _(
                                "Select a default invoice template first in the global preferences"
                            ),
                            style="color: #FF5500",
                        ),
                    ),
                ),
                layout.datatable.DataTableColumn(
                    _("Receipt"),
                    hg.If(
                        hg.F(
                            lambda c: models.pref("default_receipt_template")
                            is not None
                        ),
                        hg.If(
                            hg.C("row.payed"),
                            layout.button.Button(
                                notext=True,
                                icon="download",
                                buttontype="ghost",
                                small=True,
                            ).as_href(hg.F(receipt_download)),
                        ),
                        hg.SPAN(
                            _(
                                "Select a default receipt template first in the global preferences"
                            ),
                            style="color: #FF5500",
                        ),
                    ),
                ),
            ],
            rowactions=[
                views.BrowseView.editlink(),
                layout.modal.Modal.with_ajax_content(
                    hg.C("row"),
                    utils.ModelHref(hg.C("row"), "cancel", query={"asajax": True}),
                    submitlabel=_("Cancel"),
                    id=hg.format("cancel-{}", hg.C("row").id),
                ).with_trigger(
                    layout.button.Button.from_link(
                        utils.Link(
                            href="",
                            label=_("Cancel"),
                            iconname="trash-can",
                            permissions=[
                                hg.F(lambda c: utils.permissionname(c["row"], "delete"))
                            ],
                        ),
                        notext=True,
                        small=True,
                        buttontype="ghost",
                    )
                ),
            ],
        ),
        editview=views.EditView._with(
            fields=[invoice_form],
            get_success_url=lambda s: s.request.get_full_path(),
        ),
        addview=views.AddView._with(
            fields=[invoice_form],
            get_success_url=lambda s: utils.reverse_model(
                models.Invoice, "edit", kwargs={"pk": s.object.pk}
            ),
        ),
        markpayed=views.EditView._with(
            fields=["payed"], get_initial=lambda s: {"payed": now().date}
        ),
        cancel=views.EditView._with(
            fields=["cancelled"],
            get_initial=lambda s: {"cancelled": datetime.date.today()},
        ),
        sendinvoice=send_invoice,
        sendreceipt=send_receipt,
    ),
    *utils.default_model_paths(models.PaymentType),
]

invoicingGroup = menu.Group(_("Invoicing"))

menu.registeritem(
    menu.Item(
        utils.Link(
            utils.ModelHref(models.Invoice, "browse"),
            models.Invoice._meta.verbose_name_plural,
        ),
        invoicingGroup,
    )
)
menu.registeritem(
    menu.Item(
        utils.Link(
            utils.ModelHref(models.PaymentType, "browse"),
            models.PaymentType._meta.verbose_name_plural,
        ),
        invoicingGroup,
    )
)



import htmlgenerator as hg
from basxbread import layout
from basxbread.layout import ObjectFieldLabel, ObjectFieldValue
from basxbread.layout.components.icon import Icon
from basxbread.layout.components.tag import Tag
from django.utils.translation import gettext_lazy as _

from basxconnect.core.layouts.editperson.common import addresses, utils
from basxconnect.core.layouts.editperson.common.utils import (
    open_modal_popup_button,
    tile_with_icon,
)

R = layout.grid.Row


def common_tiles():
    return hg.BaseElement(
        R(
            tags(),
            addresses.email(),
        ),
        R(
            addresses.postals(),
            addresses.numbers(),
        ),
        R(
            other(),
            addresses.urls(),
        ),
    )


def other():
    return utils.tile_with_icon(
        Icon("add-comment"),
        hg.H4(_("Other")),
        hg.DIV(
            ObjectFieldLabel("remarks"), style="font-weight:bold; margin-bottom: 1rem;"
        ),
        ObjectFieldValue("remarks"),
        utils.open_modal_popup_button(
            "Remarks",
            hg.C("object").person_ptr,
            "ajax_edit_remarks",
        ),
    )


def tags():
    return tile_with_icon(
        Icon("tag--group"),
        hg.H4(_("Tags")),
        hg.Iterator(hg.F(lambda c: c["object"].tags.all()), "i", Tag(hg.C("i"))),
        open_modal_popup_button(
            _("Edit Tags"),
            hg.C("object").person_ptr,
            "ajax_edit_tags",
        ),
    )



# Generated by Django 3.2.5 on 2021-09-28 08:12

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("core", "0013_auto_20210928_1445"),
    ]

    operations = [
        migrations.RenameField(
            model_name="term",
            old_name="category",
            new_name="vocabulary",
        ),
        migrations.AlterField(
            model_name="email",
            name="status",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "addressstatus"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="status_core_email_list",
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="email",
            name="type",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "emailtype"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="type_core_email_list",
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="fax",
            name="status",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "addressstatus"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="status_core_fax_list",
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="fax",
            name="type",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "phonetype"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="historicallegalperson",
            name="type",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"vocabulary__slug": "legaltype"},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="historicalnaturalperson",
            name="form_of_address",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"vocabulary__slug": "form_of_address"},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="historicalnaturalperson",
            name="gender",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"vocabulary__slug": "gender"},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="historicalnaturalperson",
            name="salutation",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"vocabulary__slug": "salutation"},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="historicalnaturalperson",
            name="title",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"vocabulary__slug": "title"},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="historicalnaturalperson",
            name="type",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"vocabulary__slug": "naturaltype"},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="historicalpersonassociation",
            name="type",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"vocabulary__slug": "associationtype"},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="legalperson",
            name="type",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "legaltype"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="naturalperson",
            name="form_of_address",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "form_of_address"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="naturalperson",
            name="gender",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "gender"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="gender_persons",
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="naturalperson",
            name="salutation",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "salutation"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="naturalperson",
            name="title",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "title"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="title_persons",
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="naturalperson",
            name="type",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "naturaltype"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="person",
            name="categories",
            field=models.ManyToManyField(
                blank=True,
                limit_choices_to={"vocabulary__slug": "category"},
                to="core.Term",
            ),
        ),
        migrations.AlterField(
            model_name="personassociation",
            name="type",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "associationtype"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="phone",
            name="status",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "addressstatus"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="status_core_phone_list",
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="phone",
            name="type",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "phonetype"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="postal",
            name="status",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "addressstatus"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="status_core_postal_list",
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="postal",
            name="type",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "addresstype"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="type_core_postal_list",
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="web",
            name="status",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "addressstatus"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="status_core_web_list",
                to="core.term",
            ),
        ),
        migrations.AlterField(
            model_name="web",
            name="type",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"vocabulary__slug": "urltype"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="type_core_web_list",
                to="core.term",
            ),
        ),
    ]



# Generated by Django 3.2.4 on 2021-11-19 13:01

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("core", "0023_auto_20211030_0955"),
        ("mailer_integration", "0013_mailingpreferences_latest_sync"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="MailingPreferences",
            new_name="Subscription",
        ),
    ]



# Generated by Django 3.2.5 on 2021-09-28 08:15

from django.db import migrations
from django.utils.translation import gettext_lazy as _


class Migration(migrations.Migration):
    def rename(apps, schema_editor):
        Vocabulary = apps.get_model("core.Vocabulary")
        tag = Vocabulary.objects.filter(slug="category").first()
        if tag is not None:
            tag.slug = "tag"
            tag.name = _("Tag")
            tag.save()

    def rename_undo(apps, schema_editor):
        Vocabulary = apps.get_model("core.Vocabulary")
        tag = Vocabulary.objects.filter(slug="tag").first()
        if tag is not None:
            tag.slug = "category"
            tag.slug = _("Category")
            tag.save()

    dependencies = [
        ("core", "0014_auto_20210928_1512"),
    ]

    operations = [migrations.RunPython(rename, rename_undo)]



import datetime
import decimal
import re

import htmlgenerator as hg
from basxbread import utils
from django.db import models
from django.utils.translation import gettext_lazy as _
from djmoney.money import Money
from djmoney.settings import CURRENCY_CHOICES
from dynamic_preferences.registries import global_preferences_registry

from basxconnect.core.models import Person
from basxconnect.projects.models import Project


def pref(name, section="invoicing"):
    global_preferences = global_preferences_registry.manager()
    return global_preferences[f"{section}__{name}"]


class PaymentType(models.Model):
    name = models.CharField(
        _("Name"), max_length=255, help_text=_("Internal name of this payment method")
    )
    payment_information = models.TextField(
        _("Payment information"),
        help_text=_("Instructions for the client"),
    )

    def __str__(self):
        return self.name

    class Meta:
        ordering = ["name"]
        verbose_name = _("Payment type")
        verbose_name_plural = _("Payment types")


class InvoiceManager(models.Manager):
    def get_active(self):
        return super().get_queryset().filter(cancelled=None)

    def get_cancelled(self):
        return super().get_queryset().exclude(cancelled=None)


def default_invoice_number():
    counts = {}
    group_regex = re.compile(pref("invoice_number_grouping_regex"))
    counter_regex = re.compile(pref("invoice_number_extraction_regex"))
    for invoice in Invoice.objects.all():
        group_match = group_regex.match(getattr(invoice, "number", ""))
        counter_match = counter_regex.match(getattr(invoice, "number", ""))
        if group_match is not None and counter_match is not None:
            group = group_match.groups()[0]
            if group not in counts:
                counts[group] = 0
            try:
                counts[group] = max(int(counter_match.groups()[0]), counts[group])
            except ValueError:
                pass

    counter_next = 1
    tmp_value = utils.jinja_render(pref("invoice_number_template"), counter_next=0)
    group_match = group_regex.match(tmp_value)
    if group_match is not None:
        counter_next = counts.get(group_match.groups()[0], 0) + 1
    return utils.jinja_render(
        pref("invoice_number_template"), counter_next=counter_next
    )


def default_currency():
    return pref("default_currency")


def default_include_vat():
    return pref("amounts_include_vat")


def default_vat():
    if pref("default_use_vat") is False:
        return decimal.Decimal(0)
    return pref("default_vat")


def default_wht():
    if pref("default_use_wht") is False:
        return decimal.Decimal(0)
    return pref("default_wht")


class Invoice(models.Model):
    number = models.CharField(
        _("Invoice number"), max_length=255, default=default_invoice_number, unique=True
    )
    client = models.ForeignKey(
        Person,
        on_delete=models.CASCADE,
        related_name="invoices",
        verbose_name=_("Client"),
    )
    currency = models.CharField(
        _("Currency"), max_length=3, choices=CURRENCY_CHOICES, default=default_currency
    )
    paymenttype = models.ForeignKey(
        PaymentType,
        null=True,
        on_delete=models.SET_NULL,
        verbose_name=PaymentType._meta.verbose_name,
    )
    created = models.DateField(_("Created on"), default=datetime.date.today)
    payed = models.DateField(_("Payed on"), null=True, blank=True)
    invoice_sent = models.DateField(_("Invoice sent"), null=True, blank=True)
    receipt_sent = models.DateField(_("Receipt sent"), null=True, blank=True)
    cancelled = models.DateField(_("Cancelled"), null=True, blank=True)
    note = models.TextField(
        _("Note to client"),
        blank=True,
        help_text=_("Can be displayed on invoice and/or receipt"),
    )

    # tax related
    amounts_include_vat = models.BooleanField(
        _("Amounts include VAT"), default=default_include_vat
    )
    vat = models.DecimalField(
        _("VAT"), max_digits=4, decimal_places=3, default=default_vat
    )
    wht = models.DecimalField(
        _("WHT"), max_digits=4, decimal_places=3, default=default_wht
    )

    objects = InvoiceManager()

    def has_vat(self):
        return self.vat > 0

    has_vat.verbose_name = _("Has VAT")

    def has_wht(self):
        return self.wht > 0

    has_wht.verbose_name = _("Has WHT")

    def net_amount(self):
        net_multiply = 1
        if self.amounts_include_vat:
            net_multiply += self.vat
        amount = sum(i.amount / net_multiply for i in self.items.all())
        return Money(amount, self.currency)

    net_amount.verbose_name = _("Net amount")

    def vat_amount(self):
        return self.net_amount() * self.vat

    vat_amount.verbose_name = _("VAT amount")

    def wht_amount(self):
        return self.net_amount() * self.wht

    wht_amount.verbose_name = _("WHT amount")

    def total_amount(self):
        return (self.net_amount() + self.vat_amount()) - self.wht_amount()

    total_amount.verbose_name = _("Total amount")

    def __html__(self):
        if self.cancelled:
            return hg.SPAN(
                f"{self._meta.verbose_name} {self.number} ({self.client})",
                style="text-decoration: line-through;",
            )
        return f"{self._meta.verbose_name} {self.number} ({self.client})"

    def __str__(self):
        if self.cancelled:
            return f"{self._meta.verbose_name} {self.number} (_('Cancelled'))"
        return f"{self._meta.verbose_name} {self.number}"

    class Meta:
        ordering = ["-created", "-id"]
        verbose_name = _("Invoice")
        verbose_name_plural = _("Invoices")


def project_choices(field, request, instance):
    if instance is None or instance.client is None:
        return []
    return instance.client.projects.all()


class Item(models.Model):
    invoice = models.ForeignKey(Invoice, on_delete=models.CASCADE, related_name="items")
    amount = models.DecimalField(max_digits=14, decimal_places=2)
    description = models.CharField(max_length=1024)
    project = models.ForeignKey(
        Project,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="invoices",
        verbose_name=Project._meta.verbose_name,
    )
    project.lazy_choices = project_choices

    def amount_with_currency(self):
        return Money(self.amount, self.invoice.currency)

    def __str__(self):
        return f"{self.description}: {Money(self.amount, self.invoice.currency)}: "

    class Meta:
        verbose_name = _("Item")
        verbose_name_plural = _("Items")



# Generated by Django 3.2.10 on 2021-12-23 12:48

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("core", "0026_auto_20211216_0946"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicallegalperson",
            name="active",
            field=models.BooleanField(
                default=True, editable=False, verbose_name="Active"
            ),
        ),
        migrations.AlterField(
            model_name="historicallegalperson",
            name="default_sorting_name",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="Default Sorting Name",
            ),
        ),
        migrations.AlterField(
            model_name="historicallegalperson",
            name="deleted",
            field=models.BooleanField(
                blank=True, default=False, editable=False, verbose_name="Deleted"
            ),
        ),
        migrations.AlterField(
            model_name="historicalnaturalperson",
            name="active",
            field=models.BooleanField(
                default=True, editable=False, verbose_name="Active"
            ),
        ),
        migrations.AlterField(
            model_name="historicalnaturalperson",
            name="default_sorting_name",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="Default Sorting Name",
            ),
        ),
        migrations.AlterField(
            model_name="historicalnaturalperson",
            name="deleted",
            field=models.BooleanField(
                blank=True, default=False, editable=False, verbose_name="Deleted"
            ),
        ),
        migrations.AlterField(
            model_name="historicalperson",
            name="active",
            field=models.BooleanField(
                default=True, editable=False, verbose_name="Active"
            ),
        ),
        migrations.AlterField(
            model_name="historicalperson",
            name="default_sorting_name",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="Default Sorting Name",
            ),
        ),
        migrations.AlterField(
            model_name="historicalperson",
            name="deleted",
            field=models.BooleanField(
                blank=True, default=False, editable=False, verbose_name="Deleted"
            ),
        ),
        migrations.AlterField(
            model_name="historicalpersonassociation",
            name="active",
            field=models.BooleanField(
                default=True, editable=False, verbose_name="Active"
            ),
        ),
        migrations.AlterField(
            model_name="historicalpersonassociation",
            name="default_sorting_name",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="Default Sorting Name",
            ),
        ),
        migrations.AlterField(
            model_name="historicalpersonassociation",
            name="deleted",
            field=models.BooleanField(
                blank=True, default=False, editable=False, verbose_name="Deleted"
            ),
        ),
        migrations.AlterField(
            model_name="person",
            name="active",
            field=models.BooleanField(
                default=True, editable=False, verbose_name="Active"
            ),
        ),
        migrations.AlterField(
            model_name="person",
            name="default_sorting_name",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=255,
                verbose_name="Default Sorting Name",
            ),
        ),
        migrations.AlterField(
            model_name="person",
            name="deleted",
            field=models.BooleanField(
                blank=True, default=False, editable=False, verbose_name="Deleted"
            ),
        ),
    ]



# Generated by Django 3.2.4 on 2021-11-22 09:53

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("mailer_integration", "0015_synchronizationperson_message"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="synchronizationperson",
            name="successfully_added",
        ),
    ]



import csv
import os
import traceback

import chardet
import dateparser
import htmlgenerator as hg
from basxbread import layout as _layout
from basxbread.utils.urls import reverse_model
from basxbread.views import BaseView, generate_wizard_form
from django import forms
from django.conf import settings
from django.contrib import messages
from django.contrib.auth.mixins import PermissionRequiredMixin
from django.core.exceptions import ValidationError
from django.core.files.storage import FileSystemStorage
from django.shortcuts import redirect
from django.utils.html import mark_safe
from django.utils.translation import get_language
from django.utils.translation import gettext_lazy as _
from djmoney.contrib.exchange.models import convert_money
from dynamic_preferences.registries import global_preferences_registry
from formtools.wizard.views import NamedUrlSessionWizardView
from moneyed import Money

from basxconnect.core.models import Person

from .. import models

global_preferences = global_preferences_registry.manager()

# specifies whith model field must be taken from which column
# the value None means ignore the field
DEFAULT_COLUMN_MAPPING = {
    "date": 0,
    "note": 1,
    "debitaccount": 2,
    "creditaccount": 3,
    "donornumber": 4,
    "amount": 5,
    "currency": None,
}


def contributions_from_csv(filedata, has_header, filter_duplicates, delimiter=";"):
    global_preferences = global_preferences_registry.manager()
    mapping = settings.BASXCONNECT.get(
        "CONTRIBUTIONS_CSV_COLUMN_MAPPING", DEFAULT_COLUMN_MAPPING
    )
    if not all([key in mapping for key in DEFAULT_COLUMN_MAPPING.keys()]):
        raise ValueError(
            "Settings CONTRIBUTIONS_CSV_COLUMN_MAPPING needs "
            f"all keys {DEFAULT_COLUMN_MAPPING.keys()}"
        )

    encoding = chardet.detect(filedata).get("encoding") or "utf8"
    data = filedata.decode(encoding).splitlines()
    if has_header:
        data = data[1:]

    ret = []
    for row in csv.reader(data, delimiter=delimiter):
        if len(row) == 0:
            continue
        data = {k: row[mapping[k]] for k in mapping.keys() if mapping[k] is not None}
        data.setdefault("currency", global_preferences["contributions__currency"])
        data["date"] = dateparser.parse(data["date"], languages=[get_language()])

        externalnumber = data.pop("donornumber")
        person = Person.objects.filter(personnumber=externalnumber)
        if person.exists():
            data["person"] = person.get()
        contribution = models.Contribution(**data)
        contribution.full_clean(exclude=["_import", "person"])
        contribution.donornumber = externalnumber
        if not (
            filter_duplicates
            and models.Contribution.objects.filter(
                date=contribution.date,
                amount=contribution.amount,
                currency=contribution.currency,
                person=contribution.person,
            ).exists()
        ):
            ret.append(contribution)
    return ret


class UploadForm(forms.Form):
    title = _("Upload a file")
    importfile = forms.FileField(
        label=_("File to import"), help_text=_("Choose a CSV file"), required=False
    )
    first_line_is_header = forms.BooleanField(
        label=_("First line is header"), initial=True, required=False
    )
    filter_duplicates = forms.BooleanField(
        label=_("Filter out duplicates"), required=False
    )
    delimiter = forms.CharField(label=("Delimiter"), max_length=1, initial=";")
    layout = hg.BaseElement(
        _layout.forms.FormField("importfile"),
        _layout.helpers.Label(_("Import options")),
        _layout.forms.FormField("first_line_is_header"),
        _layout.forms.FormField("filter_duplicates"),
        _layout.forms.FormField("delimiter"),
    )

    def clean(self):
        ret = super().clean()
        if "importfile" not in ret or not ret["importfile"]:
            raise ValidationError(_("Required field"))
        try:
            contributions_from_csv(
                ret["importfile"].read(),
                ret["first_line_is_header"],
                ret["filter_duplicates"],
                ret.get("delimiter") or ";",
            )
        except Exception as e:
            traceback.print_exception(type(e), e, e.__traceback__)
            if isinstance(e, forms.ValidationError):
                msg = ["<ul>"]
                for erroritem in e:
                    if isinstance(erroritem, tuple):
                        msg.extend([f"<li>{i}</li>" for i in erroritem[1]])
                    else:
                        msg.append(f"<li>{erroritem}</li>")

                msg = mark_safe("".join(msg + ["</ul>"]))
            else:
                msg = str(e)
            raise forms.ValidationError(mark_safe(_("CSV import problem: ") + msg))
        return ret


class AssignmentForm(forms.Form):
    title = _("Assignment")
    layout = hg.BaseElement(
        _layout.datatable.DataTable(
            columns=(
                _layout.datatable.DataTableColumn(_("Date"), hg.C("row.date"), None),
                _layout.datatable.DataTableColumn(_("Note"), hg.C("row.note"), None),
                _layout.datatable.DataTableColumn(
                    _("Account"), hg.C("row.debitaccount"), None
                ),
                _layout.datatable.DataTableColumn(
                    _("Cost Center"), hg.C("row.creditaccount"), None
                ),
                _layout.datatable.DataTableColumn(
                    _("Person Number"), hg.C("row.person.personnumber"), None
                ),
                _layout.datatable.DataTableColumn(
                    _("Donor Number"), hg.C("row.donornumber"), None
                ),
                _layout.datatable.DataTableColumn(
                    _("Assignment state"),
                    hg.If(
                        hg.C("row.person"),
                        _layout.icon.Icon(
                            "checkmark--filled",
                            size=16,
                            style="fill: currentColor; color: green;",
                        ),
                        _layout.icon.Icon(
                            "warning",
                            size=16,
                            style="fill: currentColor; color: red;",
                        ),
                    ),
                    None,
                ),
                _layout.datatable.DataTableColumn(
                    _("Amount"), hg.C("row.amount_formatted"), None
                ),
            ),
            row_iterator=hg.C("contributions"),
        ).with_toolbar(_("Overview of contributions to import"), hg.C("importfile")),
        hg.DIV(
            hg.DIV(
                hg.DIV(
                    hg.F(lambda c: len(c["contributions"])),
                    " ",
                    _("contributions"),
                    _class="bx--batch-summary",
                ),
                hg.DIV(
                    _("Sum"),
                    hg.SPAN("|", style="margin-left: 1rem; margin-right: 1rem"),
                    hg.F(
                        lambda c: sum(
                            [
                                convert_money(
                                    Money(contr.amount, contr.currency),
                                    global_preferences["contributions__currency"],
                                )
                                for contr in c["contributions"]
                            ]
                        )
                        or Money(0, global_preferences["contributions__currency"])
                    ),
                    style="position: absolute; right: 0; margin-right: 1rem; color: #ffffff",
                ),
                _class="bx--batch-actions--active bx--batch-actions",
            ),
            _class="bx--table-toolbar",
            style="margin-bottom: 4rem",
        ),
        hg.DIV(style="margin-bottom: 2rem"),
        hg.If(
            hg.C("unassigned_contributions"),
            _layout.notification.InlineNotification(
                hg.BaseElement(
                    hg.F(
                        lambda c: len(
                            [c for c in c.get("contributions", ()) if not c.person]
                        )
                    ),
                    _(" contributions could not been assigned"),
                ),
                _(
                    "Please make sure that each entry has contributor number "
                    "which matches with a person number and do the import again"
                ),
                kind="error",
                lowcontrast=True,
                action=(
                    _("Cancel import"),
                    "window.location = window.location.pathname + '?reset=1'",
                ),
                style="max-width: 100%",
            ),
            _layout.notification.InlineNotification(
                _("Assignment complete"),
                _("Continue in order to complete the import"),
                kind="success",
                style="max-width: 100%",
            ),
        ),
    )


# The WizardView contains mostly control-flow logic and some configuration


class ContributionsImportWizard(
    PermissionRequiredMixin, BaseView, NamedUrlSessionWizardView
):
    initial_dict = {"upload_file": {"delimiter": ";"}}
    urlparams = (("step", str),)
    file_storage = FileSystemStorage(
        location=os.path.join(settings.MEDIA_ROOT, "wizards")
    )
    permission_required = "contributions.add_contributionimport"

    form_list = [
        ("upload_file", UploadForm),
        ("assignment", AssignmentForm),
    ]
    template_name = ""

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)

        upload_file = self.get_cleaned_data_for_step("upload_file")
        context["contributions"] = ()
        if upload_file:
            context["importfile"] = upload_file.get("importfile", None)
            if context["importfile"]:
                context["importfile"].seek(0),
                context["contributions"] = contributions_from_csv(
                    context["importfile"].read(),
                    upload_file.get("first_line_is_header", True),
                    upload_file.get("filter_duplicates", False),
                    upload_file.get("delimiter") or ";",
                )

        context["unassigned_contributions"] = len(
            [c for c in context["contributions"] if not c.person]
        )
        # disable "continue" button if there are unassigned contributions
        if (
            self.steps.current == "assignment"
            and context["unassigned_contributions"] > 0
        ):
            for button in self.get_layout().filter(
                lambda element, ancestors: isinstance(element, _layout.button.Button)
                and element.attributes.get("type") == "submit"
            ):
                button.attributes["disabled"] = True
        return context

    def get_layout(self):
        return generate_wizard_form(
            self,
            _("Import contributions"),
            self.get_form().title,
            self.get_form().layout,
        )

    def done(self, form_list, **kwargs):
        context = self.get_context_data(list(form_list)[-1])
        contributionimport = models.ContributionImport.objects.create(
            importfile=context["importfile"], user=self.request.user
        )
        for contribution in context["contributions"]:
            contribution._import = contributionimport
            contribution.save()
        messages.success(
            self.request,
            _("Successfully imported %d contributions") % len(context["contributions"]),
        )
        return redirect(reverse_model(models.ContributionImport, "browse"))

    def get(self, *args, **kwargs):
        if "reset" in self.request.GET:
            self.storage.reset()
            self.storage.current_step = self.steps.first
            return redirect(self.get_step_url(self.steps.current))
        return super().get(*args, **kwargs)



# Generated by Django 3.2.5 on 2021-10-04 10:05

from django.db import migrations, models


class Migration(migrations.Migration):
    def add_slug(apps, _):
        from django.utils.text import slugify

        for term in apps.get_model("core.Term").objects.all():
            term.slug = slugify(term.term)
            n = 1
            while apps.get_model("core.Term").objects.filter(slug=term.slug).exists():
                term.slug = slugify(term.term) + str(n)
                n += 1

            term.save()

    dependencies = [
        ("core", "0017_remove_person_categories"),
    ]

    operations = [
        migrations.AddField(
            model_name="term",
            name="slug",
            field=models.CharField(
                default="", max_length=255, verbose_name="Slug", blank=True
            ),
            preserve_default=False,
        ),
        migrations.RunPython(add_slug, migrations.RunPython.noop),
    ]



# Generated by Django 4.1.7 on 2023-02-20 04:49

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("mailer_integration", "0023_auto_20220517_1700"),
    ]

    operations = [
        migrations.AddField(
            model_name="synchronizationperson",
            name="maybe_duplicate",
            field=models.BooleanField(
                default=False,
                help_text="Other person with same first and last name already exists",
                verbose_name="Possible duplicate",
            ),
        ),
    ]



# Generated by Django 3.2.4 on 2021-10-07 07:32

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("core", "0020_auto_20211005_1630"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="postal",
            name="valid_to",
        ),
        migrations.AddField(
            model_name="postal",
            name="valid_until",
            field=models.DateField(blank=True, null=True, verbose_name="Valid until"),
        ),
    ]



# Generated by Django 4.0.6 on 2022-11-03 05:12

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("core", "0032_alter_historicallegalperson_options_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalnaturalperson",
            name="maiden_name",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="Maiden Name"
            ),
        ),
        migrations.AddField(
            model_name="naturalperson",
            name="maiden_name",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="Maiden Name"
            ),
        ),
    ]



import basxbread
import htmlgenerator as hg
from basxbread import layout
from basxbread.layout.components import tag
from basxbread.layout.components.icon import Icon
from basxbread.utils import ModelHref
from django.conf import settings
from django.shortcuts import get_object_or_404
from django.utils.formats import localize
from django.utils.timezone import localtime
from django.utils.translation import gettext_lazy as _
from dynamic_preferences.registries import global_preferences_registry

from basxconnect.core import models
from basxconnect.core.layouts.editperson.common.utils import tile_with_icon
from basxconnect.core.models import Person
from basxconnect.mailer_integration.models import Interest, Subscription

R = basxbread.layout.grid.Row
C = basxbread.layout.grid.Col


def mailer_integration_tile(request):
    person = get_object_or_404(Person, pk=request.resolver_match.kwargs["pk"])
    addresses = person.core_email_list.all()
    return tile_with_icon(
        Icon("email--new"),
        hg.H4(
            _("Email Subscriptions"),
        ),
        *[_display_subscription(email) for email in addresses]
        if hasattr(person, "core_email_list") and person.core_email_list.count() > 0
        else C(_("Person has no email addresses")),
    )


def _display_subscription(email):
    if not hasattr(email, "subscription"):
        return _display_email_without_subscription(email)
    subscription = email.subscription
    modal = modal_edit_subscription(subscription)
    interests = [
        R(
            C(interest, width=6, breakpoint="lg"),
            C(
                is_interested_indicator(
                    interest in subscription.interests.all(),
                    subscription.status != "archived",
                ),
                breakpoint="lg",
            ),
            style="padding-bottom: 24px;",
        )
        for interest in Interest.objects.all()
    ]
    return hg.DIV(
        R(
            C(
                hg.SPAN(email.email, style="font-weight: bold;"),
                tag.Tag(
                    _(subscription.status),
                    tag_color=map_tag_color(subscription),
                    onclick="return false;",
                    style="margin-left: 1rem;",
                ),
                hg.SPAN(
                    subscription.get_language_display(),
                    style="padding-left:0.5rem;",
                ),
                style="margin-bottom: 1.5rem;",
            ),
        ),
        hg.If(
            subscription.status == "archived",
            R(
                C(_("Status before archiving"), width=6, breakpoint="lg"),
                C(
                    subscription.status_before_archiving,
                    breakpoint="lg",
                ),
                style="padding-bottom: 24px;",
            ),
        ),
        *interests,
        R(
            C(_("Last synchronized"), width=6, breakpoint="lg"),
            C(
                localize(
                    localtime(subscription.latest_sync.sync_completed_datetime),
                    use_l10n=settings.USE_L10N,
                )
                if hasattr(subscription.latest_sync, "sync_completed_datetime")
                else "",
                breakpoint="lg",
            ),
            style="padding-bottom: 24px;",
        ),
        R(
            C(
                layout.button.Button(
                    "Edit",
                    buttontype="tertiary",
                    icon="edit",
                    **modal.openerattributes,
                ),
                modal,
                style="margin-top: 1.5rem;margin-bottom: 3rem;",
            )
        ),
        style=hg.If(
            email.subscription.status == "archived",
            "color: #a8a8a8;",
        ),
    )


def _display_email_without_subscription(email):
    modal_add = modal_add_subscription(email)
    return hg.BaseElement(
        hg.DIV(email.email, style="font-weight: bold;"),
        hg.DIV(_("No subscription yet for "), email.email),
        layout.button.Button(
            _("Add subscription"),
            buttontype="ghost",
            icon="add",
            **modal_add.openerattributes,
        ),
        modal_add,
    )


def map_tag_color(subscription):
    mapping = {"subscribed": "green"}
    return mapping.get(subscription.status, "gray")


def is_interested_indicator(is_interested, is_active):
    if is_interested and is_active:
        color = "#198038"
        text = _("active")
    elif is_interested:
        color = "#e0e0e0"
        text = _("active")
    else:
        color = "#e0e0e0"
        text = _("inactive")
    return hg.DIV(
        hg.DIV("●", style=f"color: {color}; display: inline-block;"),
        hg.DIV(text, style="display: inline-block; padding-left: 8px"),
        style="display: inline-block;",
    )


def modal_edit_subscription(mailingpreferences):
    modal = layout.modal.Modal.with_ajax_content(
        heading=_("Edit subscription"),
        url=ModelHref(
            Subscription,
            "ajax_edit",
            kwargs={"pk": mailingpreferences.pk},
            query={"asajax": True},
        ),
        submitlabel=_("Save"),
    )
    modal[0][1].attributes["style"] = "overflow: visible"
    modal[0].attributes["style"] = "overflow: visible"
    return modal


def modal_add_subscription(email: models.Email):
    preferred_language = email.person.preferred_language
    set_language = (
        global_preferences_registry.manager()["mailchimp__synchronize_language"]
        and preferred_language
    )
    disable_interests = global_preferences_registry.manager()[
        "mailchimp__disable_interests"
    ]
    ret = layout.modal.Modal.with_ajax_content(
        heading=_("Add subscription"),
        url=ModelHref(
            Subscription,
            "ajax_add",
            query={
                "asajax": True,
                "email": email.pk,
                "status": "subscribed",
                **({"language": preferred_language} if set_language else {}),
                **({"interests": ""} if disable_interests else {}),
            },
        ),
        submitlabel=_("Save"),
    )
    ret[0][1].attributes["style"] = "overflow: visible"
    ret[0].attributes["style"] = "overflow: visible"
    return ret



# Generated by Django 3.2.4 on 2021-11-03 13:09

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("mailer_integration", "0010_mailingpreferences_language"),
    ]

    operations = [
        migrations.CreateModel(
            name="SynchronizationResult",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("total_synchronized_persons", models.IntegerField(null=True)),
                ("sync_completed_datetime", models.DateTimeField(null=True)),
            ],
        ),
        migrations.CreateModel(
            name="NewPerson",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("email", models.CharField(max_length=100)),
                (
                    "sync_result",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="new_persons",
                        to="mailer_integration.synchronizationresult",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="InvalidPerson",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("email", models.CharField(max_length=100)),
                (
                    "sync_result",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="invalid_new_persons",
                        to="mailer_integration.synchronizationresult",
                    ),
                ),
            ],
        ),
    ]



# Generated by Django 3.2.4 on 2021-08-23 16:38

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("mailer_integration", "0005_alter_mailingpreferences_email"),
    ]

    operations = [
        migrations.AlterField(
            model_name="mailingpreferences",
            name="status",
            field=models.CharField(
                choices=[
                    ("subscribed", "subscribed"),
                    ("unsubscribed", "unsubscribed"),
                    ("non-subscribed", "non-subscribed"),
                    ("cleaned", "cleaned"),
                ],
                max_length=50,
            ),
        ),
    ]



# Generated by Django 3.1.5 on 2021-03-09 10:50

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("core", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="person",
            name="categories",
            field=models.ManyToManyField(
                blank=True,
                limit_choices_to={"category__slug": "category"},
                to="core.Term",
            ),
        ),
    ]
